@@ -1,6 +1,8 @@
 ## utils.R: population PK/PD modeling library
 ##
-## Copyright (C) 2014 - 2016  Wenping Wang
+## Copyright (C) 2014 - 2016  Wenping Wang,
+##  Portions (c) Matt Fidler and rest of the nlmixr2 team.
+##  The authorship is always up to date on git
 ##
 ## This file is part of nlmixr2.
 ##
@@ -126,7 +128,7 @@ nsis <- function() { ## build installer...
 }
 # ########################################################################
 
-# .collectWarnings --------------------------------------------------------
+# .collectWarn --------------------------------------------------------
 #' Collect warnings and just warn once.
 #'
 #' @param expr R expression
@@ -138,7 +140,7 @@ nsis <- function() { ## build installer...
 #'     the expression and a list of warning messages
 #' @author Matthew L. Fidler
 #' @noRd
-.collectWarnings <- function(expr, lst = FALSE) {
+.collectWarn <- function(expr, lst = FALSE) {
   if (getOption("nlmixr2.collectWarnings", TRUE)) {
     ws <- c()
     this.env <- environment()
@@ -264,7 +266,12 @@ cholSE <- function(matrix, tol = (.Machine$double.eps)^(1 / 3)) {
 #' @return a list of ...
 #' @export
 nmsimplex <- function(start, fr, rho = NULL, control = list()) {
-  if (is.null(rho)) rho <- environment(fr)
+  if (!is.environment(rho)) {
+    if (!is.null(rho)) {
+      warning("improper argument for 'rho'", call.=FALSE)
+    }
+    rho <- environment(fr)
+  }
   step <- -.2 * start
 
   con <- list(maxeval = 999, reltol = 1e-6, rcoeff = 1., ecoeff = 2., ccoeff = .5, trace = FALSE)
@@ -334,3 +341,44 @@ rxModelVarsS3.nlmixr2FitCore <- function(obj) {
 rxModelVarsS3.nlmixr2FitCoreSilent <- function(obj) {
   rxode2::rxModelVars(obj$ui)
 }
+
+#' C++ implementation of Matrix's nearPD
+#'
+#' With `ensureSymmetry` it makes sure it is symmetric by applying 0.5*(t(x) + x) before using nmNearPD
+#' 
+#' @inherit Matrix::nearPD
+#'
+#' @examples
+#' 
+#' set.seed(27)
+#' m <- matrix(round(rnorm(25),2), 5, 5)
+#' m <- m + t(m)
+#' diag(m) <- pmax(0, diag(m)) + 1
+#' (m <- round(cov2cor(m), 2))
+#'     
+#' near.m <- nmNearPD(m)
+#' round(near.m, 2)
+#' norm(m - near.m) # 1.102 / 1.08
+#'     
+#' round(nmNearPD(m, only.values=TRUE), 9)
+#'     
+#' ## A longer example, extended from Jens' original,
+#' ## showing the effects of some of the options:
+#'     
+#' pr <- matrix(c(1,     0.477, 0.644, 0.478, 0.651, 0.826,
+#'                0.477, 1,     0.516, 0.233, 0.682, 0.75,
+#'                0.644, 0.516, 1,     0.599, 0.581, 0.742,
+#'                0.478, 0.233, 0.599, 1,     0.741, 0.8,
+#'                0.651, 0.682, 0.581, 0.741, 1,     0.798,
+#'                0.826, 0.75,  0.742, 0.8,   0.798, 1),
+#'                nrow = 6, ncol = 6)
+#'     
+#' nc  <- nmNearPD(pr) 
+#' 
+#' @export
+nmNearPD <- function(x, keepDiag = FALSE, do2eigen = TRUE, doDykstra = TRUE, only.values = FALSE, ensureSymmetry=!isSymmetric(x), eig.tol = 1e-6, conv.tol = 1e-7, posd.tol = 1e-8, maxit = 100L, trace = FALSE) {
+  if (ensureSymmetry) {
+    x <- 0.5 * (t(x) + x)
+  }
+  .Call(`_nlmixr2est_nmNearPD_`, x, keepDiag, do2eigen, doDykstra, only.values, eig.tol, conv.tol, posd.tol, maxit, trace)
+}
